// using libraries from https://github.com/earlephilhower/ESP8266Audio
#include <IRsend.h>
#include <IRremoteESP8266.h>
#include <IRutils.h>

#include <Arduino.h>
#ifdef ESP32
  #include <WiFi.h>
  #include "SPIFFS.h"
#else
  #include <ESP8266WiFi.h>
#endif
#include "AudioFileSourceSPIFFS.h"
#include "AudioFileSourceID3.h"
#include "AudioGeneratorMP3.h"
#include "AudioOutputI2SNoDAC.h"
// NeoPixel Ring simple sketch (c) 2013 Shae Erisson
// Released under the GPLv3 license to match the rest of the
// Adafruit NeoPixel library

#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
#include <avr/power.h> // Required for 16 MHz Adafruit Trinket
#endif

// Which pin on the Arduino is connected to the NeoPixels?
#define PIN 14 // On Trinket or Gemma, suggest changing this to 1
#define NUMPIXELS 5 // Popular NeoPixel ring size

// When setting up the NeoPixel library, we tell it how many pixels,
// and which pin to use to send signals. Note that for older NeoPixel
// strips you might need to change the third parameter -- see the
// strandtest example for more information on possible values.
Adafruit_NeoPixel pixels(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);




char *voice_name = "/oyasumi.mp3";
const int voutPin = 33;
const int VOLT = 3.3; // 3.3Vを電源とした場合
const int ANALOG_MAX = 4096; // ESP32の場合
int light_status = 1;
const int motion_input = 17;
bool motion_detaected = 0;
unsigned long previousMillis = 0;

const uint16_t kIrLed = 16;
IRsend irsend(kIrLed);   

AudioGeneratorMP3 *mp3;
AudioFileSourceSPIFFS *file;
AudioOutputI2S *out;
AudioFileSourceID3 *id3;


void IRAM_ATTR button_pushed()
{ detachInterrupt(motion_input);
  motion_detaected = 1;
  }


// Called when a metadata event occurs (i.e. an ID3 tag, an ICY block, etc.
void MDCallback(void *cbData, const char *type, bool isUnicode, const char *string)
{
  (void)cbData;
  Serial.printf("ID3 callback for: %s = '", type);

  if (isUnicode) {
    string += 2;
  }
  
  while (*string) {
    char a = *(string++);
    if (isUnicode) {
      string++;
    }
    Serial.printf("%c", a);
  }
  Serial.printf("'\n");
  Serial.flush();
}

void mp3begin(){
  file = new AudioFileSourceSPIFFS(voice_name);
  id3 = new AudioFileSourceID3(file);
  id3->RegisterMetadataCB(MDCallback, (void*)"ID3TAG");
  //out = new AudioOutputI2SNoDAC();
  out = new AudioOutputI2S(0,AudioOutputI2S::INTERNAL_DAC);
  mp3 = new AudioGeneratorMP3();
  mp3->begin(id3, out);
}

void setup()
{
  WiFi.mode(WIFI_OFF); 
  Serial.begin(115200);
  delay(1000);
  pinMode(motion_input,INPUT);
  //attachInterrupt(motion_input, button_pushed, RISING);
    pixels.clear();
    pixels.setPixelColor(0,pixels.Color(50, 50, 0));
    pixels.setPixelColor(1,pixels.Color(50, 50, 0));
    pixels.setPixelColor(2,pixels.Color(50, 50, 0));
    pixels.setPixelColor(3,pixels.Color(50, 50, 0));
    pixels.setPixelColor(4,pixels.Color(50, 50, 0));
    pixels.show(); 
  irsend.begin();
  delay(1000);
  pixels.begin(); // INITIALIZE NeoPixel strip object (REQUIRED)
  SPIFFS.begin();
  Serial.printf("Sample MP3 playback begins...\n");
  mp3begin();
  mp3->stop();
}

void loop()
{
  if (mp3->isRunning()) {
    if (!mp3->loop()) mp3->stop();
  } else {
    Serial.printf("MP3 done\n");
    delay(1000);
    //char *voice_name = "/manul.mp3";
    //mp3begin();
     // R1の電圧を取得
  int reading = analogRead(voutPin);
  // AD値をmVに変換
  float voltage = ((long)reading * VOLT * 1000) / ANALOG_MAX;
  // 電圧から電流を求める I=E/R (R=1000)
  float microamp = (voltage * 1000) / 1000;
 // 電流をlxに変換
  float lx = microamp / (290 / 100);

  //Serial.print(lx);
  //Serial.println(" lx");
  delay(100);
  Serial.print(lx);
  Serial.println(" lx");
/*
  if (lx<200&&light_status == 1){
    Serial.print(lx);
    Serial.println(" lx");
    pixels.clear();
    pixels.setPixelColor(0,pixels.Color(150, 50, 0));
    pixels.setPixelColor(1,pixels.Color(150, 50, 0));
    pixels.setPixelColor(2,pixels.Color(150, 50, 0));
    pixels.setPixelColor(3,pixels.Color(150, 50, 0));
    pixels.show(); 
    irsend.send(NEC, 0x41B6659A, 32);　//全点灯
    delay(100);
    voice_name = "/oyasumi.mp3";
    mp3begin();
    light_status = 0;
  }
  */
  unsigned long currentMillis = millis();
  
     if (light_status == 1){
 attachInterrupt(motion_input, button_pushed, RISING);
  }

  if (light_status == 1){
    pixels.clear();
    pixels.setPixelColor(0,pixels.Color(20, 0, 50));
    pixels.setPixelColor(1,pixels.Color(20, 0, 50));
    pixels.setPixelColor(2,pixels.Color(20, 0, 50));
    pixels.setPixelColor(3,pixels.Color(20, 0, 50));
    pixels.setPixelColor(4,pixels.Color(20, 0, 50));
    pixels.show(); 
  }
  
  if(motion_detaected == 1&&currentMillis - previousMillis >= 30000){
  if (light_status == 1){
   delay(100);
   Serial.printf("motion_detected");
   previousMillis = currentMillis;
   pixels.clear();
    pixels.setPixelColor(0,pixels.Color(50, 0, 0));
    pixels.setPixelColor(1,pixels.Color(50, 0, 0));
    pixels.setPixelColor(2,pixels.Color(50, 0, 0));
    pixels.setPixelColor(3,pixels.Color(50, 0, 0));
    pixels.setPixelColor(4,pixels.Color(50, 0, 0));
    pixels.show(); 
   voice_name = "/nanishiteruno.mp3";
   mp3begin();
   delay(100);
   motion_detaected = 0;
       }
  }
  
  
  if (lx<700&&light_status == 1){
    Serial.print(lx);
    Serial.println(" lx");
    detachInterrupt(motion_input);
    pixels.clear();
    pixels.setPixelColor(0,pixels.Color(20, 10, 0));
    pixels.setPixelColor(1,pixels.Color(20, 10, 0));
    pixels.setPixelColor(2,pixels.Color(20, 10, 0));
    pixels.setPixelColor(3,pixels.Color(20, 10, 0));
    pixels.setPixelColor(4,pixels.Color(20, 10, 0));
    pixels.show(); 
    light_status = 0;
    delay(100);
    voice_name = "/oyasumi.mp3";
    mp3begin();
    
    /*irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    */
  }
  if (lx>700&&light_status == 0){
    Serial.print(lx);
    Serial.println(" lx");
    detachInterrupt(motion_input);
    pixels.clear();
    pixels.setPixelColor(0,pixels.Color(20, 0, 50));
    pixels.setPixelColor(1,pixels.Color(20, 0, 50));
    pixels.setPixelColor(2,pixels.Color(20, 0, 50));
    pixels.setPixelColor(3,pixels.Color(20, 0, 50));
    pixels.setPixelColor(4,pixels.Color(20, 0, 50));
    pixels.show(); 
    delay(100);
    light_status = 1;
    voice_name = "/ohayou.mp3";
    mp3begin();
    
    /*irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    delay(100);
    irsend.send(NEC,0x41B6659A,32);//常夜灯
    */
  }
  }
  

  
}
